@@ -53,12 +53,14 @@ class LlamaState:
     def __init__(
         self,
         eval_tokens: Deque[llama_cpp.llama_token],
-        eval_logits: Deque[List[float]],
+        eval_logits: Deque[List[llama_cpp.c_float]],
         llama_state,
+        llama_state_size: llama_cpp.c_size_t,
     ):
         self.eval_tokens = eval_tokens
         self.eval_logits = eval_logits
         self.llama_state = llama_state
+        self.llama_state_size = llama_state_size
 
 
 class Llama:
@@ -394,7 +396,7 @@ class Llama:
             and tuple(self.eval_tokens) == tuple(tokens[: len(self.eval_tokens)])
         ):
             if self.verbose:
-                print("generate cache hit", file=sys.stderr)
+                print("Llama.generate: cache hit", file=sys.stderr)
             reset = False
             tokens = tokens[len(self.eval_tokens) :]
 
@@ -516,7 +518,7 @@ class Llama:
 
         if self.cache and prompt_tokens in self.cache:
             if self.verbose:
-                print("cache hit", file=sys.stderr)
+                print("Llama._create_completion: cache hit", file=sys.stderr)
             self.load_state(self.cache[prompt_tokens])
 
         finish_reason = "length"
@@ -536,7 +538,7 @@ class Llama:
             if self.cache and len(completion_tokens) == 0:
                 if prompt_tokens not in self.cache:
                     if self.verbose:
-                        print("cache miss", file=sys.stderr)
+                        print("Llama._create_completion: cache miss", file=sys.stderr)
                     self.cache[prompt_tokens] = self.save_state()
 
             completion_tokens.append(token)
@@ -637,7 +639,7 @@ class Llama:
                 self.detokenize([token]).decode("utf-8", errors="ignore")
                 for token in all_tokens
             ]
-            all_logprobs = [Llama._logits_to_logprobs(row) for row in self.eval_logits]
+            all_logprobs = [Llama.logits_to_logprobs(list(map(float, row))) for row in self.eval_logits]
             for token, token_str, logprobs_token in zip(
                 all_tokens, all_token_strs, all_logprobs
             ):
@@ -950,19 +952,25 @@ class Llama:
         assert self.ctx is not None
         state_size = llama_cpp.llama_get_state_size(self.ctx)
         llama_state = (llama_cpp.c_uint8 * int(state_size))()
-        if llama_cpp.llama_copy_state_data(self.ctx, llama_state) != state_size:
+        n_bytes = llama_cpp.llama_copy_state_data(self.ctx, llama_state)
+        if int(n_bytes) > int(state_size):
             raise RuntimeError("Failed to copy llama state data")
+        llama_state_compact = (llama_cpp.c_uint8 * int(n_bytes))()
+        llama_cpp.ctypes.memmove(llama_state_compact, llama_state, int(n_bytes))
+        if self.verbose:
+            print(f"Llama.save_state: saving {n_bytes} bytes of llama state", file=sys.stderr)
         return LlamaState(
             eval_tokens=self.eval_tokens.copy(),
             eval_logits=self.eval_logits.copy(),
-            llama_state=llama_state,
+            llama_state=llama_state_compact,
+            llama_state_size=n_bytes,
         )
 
     def load_state(self, state: LlamaState) -> None:
         assert self.ctx is not None
         self.eval_tokens = state.eval_tokens.copy()
         self.eval_logits = state.eval_logits.copy()
-        state_size = llama_cpp.llama_get_state_size(self.ctx)
+        state_size = state.llama_state_size
         if llama_cpp.llama_set_state_data(self.ctx, state.llama_state) != state_size:
             raise RuntimeError("Failed to set llama state data")
 
@@ -977,7 +985,7 @@ class Llama:
         return llama_cpp.llama_token_bos()
 
     @staticmethod
-    def logits_to_logprobs(logits: List[llama_cpp.c_float]) -> List[llama_cpp.c_float]:
+    def logits_to_logprobs(logits: List[float]) -> List[float]:
         exps = [math.exp(float(x)) for x in logits]
         sum_exps = sum(exps)
-        return [llama_cpp.c_float(math.log(x / sum_exps)) for x in exps]
+        return [math.log(x / sum_exps) for x in exps]
